import copy
import os
import sys

from ews.coordinator.processor_deposition import ProcessorDeposition
from ews.coordinator.utils.processor_utils import parse_json_file_with_tokens
from integration.partial.integration_test_utils import IntegrationTestUtils
from integration.test_suites.depo_test_suite import BaseDepoTestSuite


class FullTestDeposition(BaseDepoTestSuite.DepoTestSuite):

    def set_expected_values(self):
        self.EA_CSV_COUNT = 9
        self.ETH_CSV_COUNT = 9
        self.EA_PNG_COUNT = 3
        self.ETH_PNG_COUNT = 3


    def setUp(self) -> None:

        self.set_expected_values()

        if IntegrationTestUtils.TEST_JOB_DIR is None \
                or not os.path.exists(IntegrationTestUtils.TEST_JOB_DIR):
        # if True:
            FullTestDeposition.write_temp_run_config_file()
            FullTestDeposition.run_depo_pipeline()
        else:
            print(f"output in {IntegrationTestUtils.TEST_JOB_DIR} already written, skipping rerun")


    @staticmethod
    def write_temp_run_config_file():

        """
        IntegrationTestUtils.RUN_SYS_CONFIG_FILE_PATH will have been set from the command line args, and point to the
        production sys config file. We need to modify this file to point to the test output directory.
        :return:
        """
        sys_config = IntegrationTestUtils.RUN_SYS_CONFIG_FILE_PATH
        sys_config_dict: dict = parse_json_file_with_tokens(sys_config)

        sys_config_dict['WorkspacePathout'] = IntegrationTestUtils.TEST_OUT_PATH
        sys_config_dict['WorkspacePath'] = IntegrationTestUtils.TEST_OUT_PATH

        os.makedirs(IntegrationTestUtils.TEST_OUT_PATH, exist_ok = True)
        job_sys_config_file = IntegrationTestUtils.TEST_OUT_PATH + "temp_sys_deposition_config.json"
        IntegrationTestUtils.write_json_file(sys_config_dict, job_sys_config_file)

        # now set the sys config file path to the new job-specific file
        IntegrationTestUtils.RUN_SYS_CONFIG_FILE_PATH = job_sys_config_file

        run_config = IntegrationTestUtils.RUN_CONFIG_FILE_PATH
        run_config_temp_dict: dict = parse_json_file_with_tokens(run_config, sys_config_dict)
        run_dict: dict = copy.deepcopy(run_config_temp_dict)

        job_run_config_file = IntegrationTestUtils.TEST_OUT_PATH + "temp_deposition_config.json"
        IntegrationTestUtils.write_json_file(run_dict, job_run_config_file)

        # set the run config file path to the new job-specific file
        IntegrationTestUtils.RUN_CONFIG_FILE_PATH = job_run_config_file


    @staticmethod
    def run_depo_pipeline():
        depo_processor = ProcessorDeposition()
        IntegrationTestUtils.run_external_pipeline(BaseDepoTestSuite.DepoTestSuite.DEPO_COMPONENT_NAME,
                                                   BaseDepoTestSuite.DepoTestSuite.DEPO_PROCESSOR_DIR,
                                                   IntegrationTestUtils.TEST_START_DATE,
                                                   depo_processor)


if __name__ == '__main__':
    _success: bool = IntegrationTestUtils.run_full_integration_test_pipeline(FullTestDeposition,
                                                                             test_prefix = "deposition",
                                                                             processor_dir = BaseDepoTestSuite.DepoTestSuite.DEPO_PROCESSOR_DIR)
    if not _success:
        sys.exit(1)
