import copy
import os
import sys

from ews.coordinator.processor_epidemiology import ProcessorEpidemiology
from ews.coordinator.utils.processor_utils import parse_json_file_with_tokens
from integration.partial.integration_test_utils import IntegrationTestUtils
from integration.test_suites.epi_test_suite import BaseEpiTestSuite


class FullTestEpi(BaseEpiTestSuite.EpiTestSuite):

    def set_expected_values(self):
        pass


    def setUp(self) -> None:

        self.set_expected_values()

        if IntegrationTestUtils.TEST_JOB_DIR is None \
                or not os.path.exists(IntegrationTestUtils.TEST_JOB_DIR):
            # if True:
            FullTestEpi.write_temp_run_config_files()
            FullTestEpi.run_epi_pipeline()
        else:
            print(f"output in {IntegrationTestUtils.TEST_JOB_DIR} already written, skipping rerun")


    @staticmethod
    def write_temp_run_config_files():
        """
        IntegrationTestUtils.RUN_SYS_CONFIG_FILE_PATH will have been set from the command line args, and point to the
        production sys config file. We need to modify this file to point to the test output directory.
        :return:
        """
        sys_config = IntegrationTestUtils.RUN_SYS_CONFIG_FILE_PATH
        sys_config_dict: dict = parse_json_file_with_tokens(sys_config)

        sys_config_dict['WorkspacePathout'] = IntegrationTestUtils.TEST_OUT_PATH
        sys_config_dict['WorkspacePath'] = IntegrationTestUtils.TEST_OUT_PATH

        os.makedirs(IntegrationTestUtils.TEST_OUT_PATH, exist_ok = True)
        job_sys_config_file = IntegrationTestUtils.TEST_OUT_PATH + "temp_sys_epi_config.json"
        IntegrationTestUtils.write_json_file(sys_config_dict, job_sys_config_file)

        # now set the sys config file path to the new job-specific file
        IntegrationTestUtils.RUN_SYS_CONFIG_FILE_PATH = job_sys_config_file

        run_config = IntegrationTestUtils.RUN_CONFIG_FILE_PATH
        run_config_temp_dict: dict = parse_json_file_with_tokens(run_config, sys_config_dict)
        run_dict: dict = copy.deepcopy(run_config_temp_dict)
        # we need to run the env suit pipeline as well
        run_dict['Environment']['WORK_PATH'] = IntegrationTestUtils.TEST_OUT_PATH
        run_dict['Environment']['INPUT_PATH'] = IntegrationTestUtils.TEST_OUT_PATH
        run_dict['Environment']['OUTPUT_PATH'] = IntegrationTestUtils.TEST_OUT_PATH

        # change the config for the Epi so it only runs on a single past date. Requires use of a different
        # FileListerFunction and a different calculation span
        run_dict['CalculationSpanDays'] = [0, 1]
        run_dict['Deposition']['FileListerFunction'] = "list_onefile_operational"
        run_dict['Environment']['FileListerFunction'] = "list_onefile_operational"

        """
        When set to true, continue mode speeds up computation time and reduces data volume by starting from the start of 
        model formulations. This requires each epidemiological model formulation described under 'Epi' to have an 
        inputs specified alongside Deposition and Environment. In this test version, we do not have prior runs to draw 
        from, so setting to false
        """
        run_dict['continue'] = False
        previous_day_string: str = IntegrationTestUtils.get_day_before_as_string(IntegrationTestUtils.TEST_START_DATE)
        run_dict['SeasonStartString'] = previous_day_string

        job_run_config_file = IntegrationTestUtils.TEST_OUT_PATH + "temp_epi_config.json"
        IntegrationTestUtils.write_json_file(run_dict, job_run_config_file)

        # set the run config file path to the new job-specific file
        IntegrationTestUtils.RUN_CONFIG_FILE_PATH = job_run_config_file


    @staticmethod
    def run_epi_pipeline():
        epi_processor = ProcessorEpidemiology()
        IntegrationTestUtils.run_external_pipeline(BaseEpiTestSuite.EpiTestSuite.EPI_COMPONENT_NAME,
                                                   BaseEpiTestSuite.EpiTestSuite.EPI_PROCESSOR_DIR,
                                                   IntegrationTestUtils.TEST_START_DATE,
                                                   epi_processor)


if __name__ == '__main__':
    _success: bool = IntegrationTestUtils.run_full_integration_test_pipeline(FullTestEpi,
                                                                             test_prefix = "epi",
                                                                             processor_dir = BaseEpiTestSuite.EpiTestSuite.EPI_PROCESSOR_DIR)
    if not _success:
        sys.exit(1)
