import copy
import os
import sys

from ews.coordinator.processor_surveys import ProcessorSurveys
from ews.coordinator.utils.processor_utils import parse_json_file_with_tokens
from integration.partial.integration_test_utils import IntegrationTestUtils
from integration.test_suites.survey_test_suite import BaseSurveyTestSuite


class FullTestSurvey(BaseSurveyTestSuite.SurveyTestSuite):

    def set_expected_values(self):
        #  no values to set
        pass


    def setUp(self) -> None:

        self.set_expected_values()

        if IntegrationTestUtils.TEST_JOB_DIR is None \
                or not os.path.exists(IntegrationTestUtils.TEST_JOB_DIR):
            FullTestSurvey.write_temp_run_config_file()
            FullTestSurvey.run_survey_pipeline()
        else:
            print(f"output in {IntegrationTestUtils.TEST_JOB_DIR} already written, skipping rerun")


    @staticmethod
    def write_temp_run_config_file():

        """
        IntegrationTestUtils.RUN_SYS_CONFIG_FILE_PATH will have been set from the command line args, and point to the
        production sys config file. We need to modify this file to point to the test output directory.
        :return:
        """
        sys_config = IntegrationTestUtils.RUN_SYS_CONFIG_FILE_PATH
        sys_config_dict: dict = parse_json_file_with_tokens(sys_config)
        sys_config_dict['WorkspacePathout'] = IntegrationTestUtils.TEST_OUT_PATH
        sys_config_dict['WorkspacePath'] = IntegrationTestUtils.TEST_OUT_PATH

        os.makedirs(IntegrationTestUtils.TEST_OUT_PATH, exist_ok = True)
        job_sys_config_file = IntegrationTestUtils.TEST_OUT_PATH + "temp_sys_survey_config.json"
        IntegrationTestUtils.write_json_file(sys_config_dict, job_sys_config_file)

        # now set the sys config file path to the new job-specific file
        IntegrationTestUtils.RUN_SYS_CONFIG_FILE_PATH = job_sys_config_file

        run_config = IntegrationTestUtils.RUN_CONFIG_FILE_PATH
        run_config_temp_dict: dict = parse_json_file_with_tokens(run_config, sys_config_dict)
        run_dict: dict = copy.deepcopy(run_config_temp_dict)
        run_dict['SkipServerDownload'] = False

        job_run_config_file = IntegrationTestUtils.TEST_OUT_PATH + "temp_survey_config.json"
        IntegrationTestUtils.write_json_file(run_dict, job_run_config_file)

        # set the run config file path to the new job-specific file
        IntegrationTestUtils.RUN_CONFIG_FILE_PATH = job_run_config_file


    @staticmethod
    def run_survey_pipeline():
        survey_processor = ProcessorSurveys()
        IntegrationTestUtils.run_external_pipeline(BaseSurveyTestSuite.SurveyTestSuite.SURVEY_COMPONENT_NAME,
                                                   BaseSurveyTestSuite.SurveyTestSuite.SURVEY_PROCESSOR_DIR,
                                                   IntegrationTestUtils.TEST_START_DATE,
                                                   survey_processor)


if __name__ == '__main__':
    _success: bool = IntegrationTestUtils.run_full_integration_test_pipeline(FullTestSurvey,
                                                                             test_prefix = "survey",
                                                                             processor_dir = BaseSurveyTestSuite.SurveyTestSuite.SURVEY_PROCESSOR_DIR)
    if not _success:
        sys.exit(1)
